/************************************************************************
 *   IRC - Internet Relay Chat, ircd/s_socks.c
 *   Copyright (C) 1998 Jonathan Chapman
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "res.h"
#include "numeric.h"
#include "patchlevel.h"
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#ifdef  UNIXPORT
# include <sys/un.h>
#endif
#if defined(__hpux)
# include "inet.h"
#endif
#include <fcntl.h>
#include "sock.h"       /* If FD_ZERO isn't define up to this point,  */
                        /* define it (BSD4.2 needs this) */
#include "h.h"

#define	USE_SOCKS

void 	check_socks(cptr)
Reg1	aClient *cptr;
{
#ifdef	USE_SOCKS
	struct sockaddr_in sock;
	int socklen = sizeof(struct sockaddr_in);

	if ((cptr->socksfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	    {
		if(!DoingDNS(cptr) && !DoingAuth(cptr))
			SetAccess(cptr);
		return;
	    }

        if (cptr->socksfd >= (MAXCONNECTIONS - 2))
            {
		sendto_ops("Can't allocate fd for socks on %s",
			get_client_name(cptr, TRUE));

		(void)close(cptr->socksfd);
		return;
	    }

	set_non_blocking(cptr->socksfd, cptr);

        getsockname(cptr->fd, (struct sockaddr *)&sock, &socklen);
        sock.sin_port = 0;
        sock.sin_family = AF_INET;

        (void)bind(cptr->socksfd, (struct sockaddr *)&sock, sizeof(sock));

        bcopy((char *)&cptr->ip, (char *)&sock.sin_addr,
                sizeof(struct in_addr));

        sock.sin_port = htons(1080);
        sock.sin_family = AF_INET;

        (void)alarm((unsigned)4);
        if (connect(cptr->socksfd, (struct sockaddr *)&sock,
                    sizeof(sock)) == -1 && errno != EINPROGRESS)
            {
                ircstp->is_abad++;
                (void)alarm((unsigned)0);
                (void)close(cptr->socksfd);
                cptr->socksfd = -1;
                if (!DoingDNS(cptr) && !DoingAuth(cptr))
                        SetAccess(cptr);
	        return;
            }
        (void)alarm((unsigned)0);

        cptr->flags2 |= (FLAGS_SOCKSW|FLAGS_SOCKS);

        if (cptr->socksfd > highest_fd)
                highest_fd = cptr->socksfd;
#endif
        return;
}

void 	send_socks(cptr)
Reg1	aClient *cptr;
{
	static unsigned char    packet[3] = {5, 1, 0};

        if (write(cptr->socksfd, packet, 3) < 3) {
                ircstp->is_abad++;
                (void)close(cptr->socksfd);
                if (cptr->socksfd == highest_fd)
                        while (!local[highest_fd])
                                highest_fd--;
                cptr->socksfd = -1;
                cptr->flags2 &= ~(FLAGS_SOCKS|FLAGS_SOCKSW);
                if (!DoingDNS(cptr) && !DoingAuth(cptr))
                        SetAccess(cptr);
                return;
            }

        cptr->flags2 &= ~FLAGS_SOCKSW;

        return;
}

void 	read_socks(cptr)
Reg1    aClient *cptr;
{
        Reg2     int     len;
        unsigned char   incoming[2];


        if (len = read(cptr->socksfd, incoming, 2) == 2)
                 if (incoming[0] == 5 && incoming[1] == 0)
                        cptr->flags2 |= FLAGS_SOCKSS;


                 if (cptr->socksfd == highest_fd)
                        while (!local[highest_fd])
                                highest_fd--;

                (void)close(cptr->socksfd);
                cptr->socksfd = -1;

                ClearSocks(cptr);

                if (!DoingDNS(cptr) && !DoingAuth(cptr))
                        SetAccess(cptr);

        return;
}

